library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity DSB_TOP is
	port(
		clk 		: 	in  STD_LOGIC;
		cs			:	out STD_LOGIC;	 
		ldac		:	out STD_LOGIC;	
		pd			:	out STD_LOGIC; 
		we			:	out STD_LOGIC;
		dac_out	:	out STD_LOGIC_VECTOR(11 downto 0)
		);
end DSB_TOP;

architecture DSB_TOP of DSB_TOP is
-- CLK DIV.
component clkdiv is
	port(
		clk 			:	in STD_LOGIC;
		clk_out		:	out STD_LOGIC;
		clk_out1		:	out STD_LOGIC;
		reset			:	out STD_LOGIC;
		clock_en    :  out STD_LOGIC
		--clk_out2		:	out STD_LOGIC
		);
end component;
-- ADC
component ADC
	port(
		clk 			: in  STD_LOGIC;
      adc_out 		: inout  STD_LOGIC_VECTOR (11 downto 0)
		);
		
end component;
-- DAC
component DAC
	port(
		clk_in 		: 	in 	STD_LOGIC;
		dac_in		:	in 	STD_LOGIC_VECTOR(11 downto 0);
		cs				:	out 	STD_LOGIC;	-- Chip select signal 
		ldac			:	out 	STD_LOGIC;	-- Load ADC 
		pd				:	out 	STD_LOGIC; -- low power mode 
		we				:	out 	STD_LOGIC;	-- write enable
		dac_out		:	out 	STD_LOGIC_VECTOR(11 downto 0)
		);
end component;
-- FILTER
component filter IS
   PORT( 
		clk         :   in    std_logic; 
      clk_enable	:   in    std_logic; 
      reset			:   in    std_logic; 
      filter_in	:   in    std_logic_vector(11 DOWNTO 0); -- sfix12_E1
      filter_out	:   out   std_logic_vector(11 DOWNTO 0)  -- sfix12_E1
		);
END component;

signal clock_en : STD_LOGIC;
signal reset : STD_LOGIC;
signal clk_out : STD_LOGIC;
signal clk_out1 : STD_LOGIC;
signal adc_out :std_logic_vector(11 DOWNTO 0); -- sfix12_E1
signal dac_in :std_logic_vector(11 DOWNTO 0); -- sfix12_E1
begin
	U1: clkdiv
		PORT MAP(
			clk => clk,
			clk_out => clk_out,
			clk_out1 => clk_out1,
			reset => reset
			--clk_out2 => clk_out2
		);
	U2: DAC
		PORT MAP(
			clk_in => clk_out, --48kHz clock
			cs => cs,
			ldac => ldac,
			pd => pd,
			we => we,
			dac_out => dac_out,
			dac_in => dac_in
		);
	U0: filter
		PORT MAP(
			clk => clk_out,
         clk_enable => clock_en,     
         reset => reset,       
         filter_in => adc_out,                
         filter_out => dac_in    
		);
	U4: ADC
		PORT MAP(
			clk => clk_out1,
			adc_out => adc_out
		);

end DSB_TOP;